//+------------------------------------------------------------------+
//|  9:19 AM 10/1/2014                                       crb.mqh |
//|                               Copyright  2014, William H Roeder |
//|                                        mailto:WHRoeder@yahoo_com |
//| Constant Range Bar chart                                         |
//+------------------------------------------------------------------+
//| CRB::Update calls user function ChartUpdated(const cChart&, bool |
//| isFinal=false) before starting a new bar with (this, true).      |
//| Caller is responsible to call ChartUpdated(this) before          |
//| retrieving any Close() dependent values.                         |
//+------------------------------------------------------------------+
#ifndef CRB_MQH
#define CRB_MQH
#include "chart.mqh" // class cChart;
class cCRB : public cChart{
 public:
   void                 Initialize(double);
   void                 Update(int iBar);
 private:
   void                 MoveTo(double, double);

   double               mSize;
   double               mLastOpen;
   double               mLastHigh;
   double               mLastLow;
}; // cCRB
//+- cCRB -------------------------------------------------- public -+
void                 cCRB::Initialize(double pips){
   mSize = pips * _Point;  if(Digits % 2 != 0)  mSize *= 10.0;
   mLastOpen = 0.0;
}
void                 cCRB::Update(int iBar){
   double   size     = mSize;
   if(size <= 0.0){
      size = iATR(NULL,0, int(MathMin(-size, Bars-iBar)), iBar);   if(size == 0.0)   size = 20.0*Point;
   }
   double   op       = Open[iBar];
   double   hi       =  High[iBar];
   double   lo       =   Low[iBar];
   double   cl       = Close[iBar];
   if(mLastOpen != op){
      if(mLastOpen == 0.0) cChart::Initialize(op, 1); // First call.
      else                 MoveTo(op, size);          // New time based bar.
      mLastOpen = op;   mLastHigh = op;   mLastLow = op;
   }
   // Must revisit hi/lo in case of disconnection during current bar.
   if(cl > op){
      if(lo != mLastLow){  MoveTo(lo, size); mLastLow    = lo; }
      if(hi != mLastHigh){ MoveTo(hi, size); mLastHigh   = hi; }
   }else{
      if(hi != mLastHigh){ MoveTo(hi, size); mLastHigh   = hi; }
      if(lo != mLastLow){  MoveTo(lo, size); mLastLow    = lo; }
   }
                           MoveTo(cl, size);
}
//+- cCRB ------------------------------------------------- private -+
void                 cCRB::MoveTo(double price, double size){
   double   hi = High();   double   loLim = hi - size;
   double   lo =  Low();   double   hiLim = lo + size;
                                                // int count=10;
   while(price > hiLim){
                                                // Print("lo("+PriceToStr(lo)
                                                // +")<h("+PriceToStr(hi)
                                                // +")<hh("+PriceToStr(hiLim)
                                                // +")<p("+PriceToStr(price)
                                                // +") s="+PriceToStr(size)
                                                // );
                                                // if(--count<0)break;
      if(hiLim > hi)             SetHigh(hiLim);   // Always unless size shrunk.
      SetClose(hiLim);           ChartUpdated(this, true);
      lo = hi = hiLim + _Point;  Insert(hi);    hiLim = lo + size;
   }
                                                   // count=10;
   while(price < loLim){
                                                   // Print(""
                                                   // +"p("+PriceToStr(price)
                                                   // +")<ll("+PriceToStr(loLim)
                                                   // +")<lo("+PriceToStr(lo)
                                                   // +")<h("+PriceToStr(hi)
                                                   // +") s="+PriceToStr(size)
                                                   // );
                                                   // if(--count<0)break;
      if(loLim < lo)                SetLow(loLim);
      SetClose(loLim);              ChartUpdated(this, true);
      lo = hi = loLim - _Point;  Insert(hi);    loLim = hi - size;
   }
   if(price > hi)    SetHigh(price);
   if(price < lo)    SetLow(price);
                     SetClose(price);
}
               // string PriceToStr(double p){return DoubleToString(p,Digits);}
//+------------------------------------------------------------------+
#endif   // CRB_MQH
